#!/usr/bin/wish
# Modified 10/2000  Clif Flynt
# Modifications Copyright (C) 2000 Clif Flynt
# clif@noucorp.com
#
# Added package and namespace support.
#
# Split child lists from Tree array to Tree_child array to reduce
#   collisions in crowded trees.  (No perceptible performance gain.)
# 
# Added newitem options:
# -text foo flag on newitem to allow directory style trees to 
#   be displayed with other identifying text.
# -bind To add a binding to an item
#
#  Added new create flags to create to modify tree behavior/display
#  -graybar 	If 1, color every other line (graybar/greenbar pattern).
#  -graycolor   If set, this is the color to use for the gray lines.
#  -windowsMode	Can open a directory with a double click (not single) on
#               text, icon, or [+]
#  -highlight	Highlights lines blue when the cursor is above them.
#  -highlightcolor Alternate color instead of blue 
#
# Added -test option to run demonstration test as standalone.
#
# Copyright (C) 1997,1998 D. Richard Hipp
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Library General Public
# License as published by the Free Software Foundation; either
# version 2 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Library General Public License for more details.
# 
# You should have received a copy of the GNU Library General Public
# License along with this library; if not, write to the
# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
# Boston, MA  02111-1307, USA.
#
# Author contact information:
#   drh@acm.org
#   http://www.hwaci.com/drh/
#
# $Revision: 2.3 $
# $Header: /home/cvs/MegaWidget/tree.tcl,v 2.3 2000/10/31 03:31:42 clif Exp $

option add *highlightThickness 0

package provide dirTree 2.0

namespace eval Tree {

namespace export create newitem

switch $tcl_platform(platform) {
  unix {
    set Tree(font) {helvetica 12 normal}
  }
  windows {
    set Tree(font) {helvetica 14 normal}
  }
}

  set Tree(fontHt) [expr [font metrics $Tree(font) -linespace] + 6]
  
  # Enabling highlight causes the text to be highlighted when the cursor 
  #  passes over it.

  set Tree(highlight) 0
  set Tree(highlightcolor) blue

  # Enabling windowsMode requires double-click to open a directory,
  #  and you can click on + or name to open.

  set Tree(windowsMode) 0

  # Setting graybar causes the tree to be drawn with every other line shaded
  set Tree(graybar) 0
  set Tree(graycolor) #ddd

# Create a new tree widget.  $args become the configuration arguments to
# the canvas widget from which the tree is constructed.

proc create {w args} {
  variable Tree

  # Set default values for these settings
  foreach k {highlight fontHt graybar graycolor windowsMode} {
      set Tree($w:$k) $Tree($k)
  }
  
  # Modify from the command line as required.
  foreach {k v} $args {
      switch -- $k {
         -graybar -
	 -fontHt  -
	 -graycolor -
	 -highlight -
	 -highlightcolor -
	 -windowsMode {
	     set k1 [string range $k 1 end]
	     set Tree($w:$k1) $v
	     set p [lsearch $args $k]
	     set p2 [expr $p + 1]
	     set args [lreplace $args $p $p2]
	 }
	 
      }
  }

  eval canvas $w -bg white $args
#  bind $w <Destroy> "Tree:delitem $w /"
  bind $w <Destroy> "catch {[namespace current]::delitem $w /}"
  dfltconfig $w /
  buildwhenidle $w
  set Tree($w:selection) {}
  set Tree($w:selidx) {}
  set Tree($w:single) 1
#  set Tree($w:background) [$w cget -background]

  return $w
}

# Initialize a element of the tree.
# Internal use only
#
proc dfltconfig {w v} {
  variable Tree
  variable Tree_children
  set Tree_children($w:$v) {}
  set Tree($w:$v:open) 0
  set Tree($w:$v:icon) {}
  set Tree($w:$v:tags) {}
}

#
# Pass configuration options to the tree widget
#
proc config {w args} {
  eval $w config $args
}

#
# Insert a new element $v into the tree $w.
#
proc newitem {w v args} {
  variable Tree
  variable Tree_children
  variable CollisionCheck
  
  if {[string match "/" $v]} {
      return
  }

  set dir [file dirname $v]
  
  set n [file tail $v]

  if {![info exists Tree($w:$dir:open)]} {
    error "parent item \"$dir\" is missing"
  }
  
  if {[info exists CollisionCheck($w:$v)]} {
    error "item \"$v\" already exists"
  } else {
    set CollisionCheck($w:$v) $n
  }

  dfltconfig $w $v

  set Tree($w:$v:text) [file tail $v]

  foreach {op arg} $args {
    switch -exact -- $op {
      -image {set Tree($w:$v:icon) $arg}
      -tags {set Tree($w:$v:tags) $arg}
      -text {set Tree($w:$v:text) $arg;}
      -bind {set Tree($w:$v:bind) $arg;}
    }
  }

#  if {![string match [info commands "sortlist"] ""]} {
#      set Tree_children($w:$dir) [sortlist insert $Tree_children($w:$dir) $n]
#  } else {
      set Tree_children($w:$dir) [listInsert $Tree_children($w:$dir) $n]
#  }


  buildwhenidle $w
}

proc listInsert {lst val} {

   set end [llength $lst]
   set start 0
   set op -1
   while {$op != [set np [expr ($start + $end) / 2]]} {
      if {[string compare [lindex $lst $np] $val] > 0} {
         set end $np
      } else {
         set start $np
      }
      set op $np
   }
   
   return [linsert $lst $end $val]
}


#
# Delete element $v from the tree $w.  If $v is /, then the widget is
# deleted.
#
proc delitem {w v} {
  variable Tree
  variable Tree_children
  variable CollisionCheck

  if {![info exists Tree($w:$v:open)]} return
  if {[string compare $v /]==0} {
    # delete the whole widget
    catch {destroy $w}
    foreach t [array names Tree $w:*] {
      unset Tree($t)
    }
    foreach t [array names CollisionCheck $w:*] {
      unset CollisionCheck($t)
    }
    return
  }
  foreach c $Tree_children($w:$v) {
    catch { delitem $w $v/$c}
  }
  catch {unset Tree($w:$v:open)}
  catch {unset Tree_children($w:$v)}
  catch {unset Tree($w:$v:icon)}
  set dir [file dirname $v]
  set n [file tail $v]
  unset CollisionCheck($w:$v)
  set i [lsearch -exact $Tree_children($w:$dir) $n]
  if {$i>=0} {
    set Tree_children($w:$dir) [lreplace $Tree_children($w:$dir) $i $i]
  }
   buildwhenidle $w
}

#
# Bitmaps used to show which parts of the tree can be opened.
#
set maskdata "#define solid_width 9\n#define solid_height 9"
append maskdata {
  static unsigned char solid_bits[] = {
   0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01, 0xff, 0x01,
   0xff, 0x01, 0xff, 0x01, 0xff, 0x01
  };
}
set data "#define open_width 9\n#define open_height 9"
append data {
  static unsigned char open_bits[] = {
   0xff, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x7d, 0x01, 0x01, 0x01,
   0x01, 0x01, 0x01, 0x01, 0xff, 0x01
  };
}
image create bitmap  openbm -data $data -maskdata $maskdata \
  -foreground black -background white
set data "#define closed_width 9\n#define closed_height 9"
append data {
  static unsigned char closed_bits[] = {
   0xff, 0x01, 0x01, 0x01, 0x11, 0x01, 0x11, 0x01, 0x7d, 0x01, 0x11, 0x01,
   0x11, 0x01, 0x01, 0x01, 0xff, 0x01
  };
}
image create bitmap  closedbm -data $data -maskdata $maskdata \
  -foreground black -background white

# Internal use only.
# Draw the tree on the canvas
proc build w {
  variable Tree
  $w delete all
  catch {unset Tree($w:buildpending)}
  set Tree($w:y) 30
  buildlayer $w / 10
  $w config -scrollregion [$w bbox all]
  drawselection $w
}

# Internal use only.
# Build a single layer of the tree on the canvas.  Indent by $in pixels
proc buildlayer {w v in {graybar 0}} {
  variable Tree
  variable Tree_children
  if {$v=="/"} {
    set vx {}
  } else {
    set vx $v
  }
  set start [expr $Tree($w:y)-10]

  set x [expr $in+12]
  set tx $x 
  
  foreach c $Tree_children($w:$v) {
    set y $Tree($w:y)

    incr Tree($w:y) $Tree($w:fontHt)
    
    # Toggle the graybar

    if {$graybar} {set graybar 0} else {set graybar 1}

    $w create line $in $y [expr $in+10] $y -fill gray50 
    set icon $Tree($w:$vx/$c:icon)
    set taglist x
    foreach tag $Tree($w:$vx/$c:tags) {
      lappend taglist $tag
    }

    if {[string length $icon]>0} {
      set k [$w create image $x $y -image $icon -anchor w -tags $taglist]
      set Tree($w:tag:$k) $vx/$c
      set tx [expr $in+ [image width $icon] + 15]
    }

    if {$Tree($w:graybar)} {
       if {$graybar} {
           set lln [$w create line 0 $y 2000 $y -width $Tree($w:fontHt) \
	       -fill $Tree($w:graycolor) ]
	   $w lower $lln
       }
    }

    set j1 [$w create text $tx $y -text $Tree($w:$vx/$c:text) \
        -font $Tree(font) \
        -anchor w -tags $taglist]

    if {$Tree($w:highlight)} {
        $w bind $j1 <Enter> "$w itemconfigure $j1 -fill $Tree($w:highlightcolor); set [namespace current]::Tree($w:highlight) $j1"
        $w bind $j1 <Leave> "$w itemconfigure $j1 -fill black; set [namespace current]::Tree($w:highlight) {}"
    }

    if {[info exists Tree($w:$vx/$c:bind)]} {
        $w bind $j1 <1> $Tree($w:$vx/$c:bind)
    }
    
    set Tree($w:tag:$j1) $vx/$c
    set Tree($w:$vx/$c:tag) $j1
    set nmsp [namespace current]
# puts "TAGLIST: $taglist"
    if {[string length $Tree_children($w:$vx/$c)]} {
      if {$Tree($w:$vx/$c:open)} {
         set j [$w create image $in $y -image  openbm -tags $taglist]
	 if {$Tree($w:windowsMode)} {
	     if {[info exists k]} {
               $w bind $k <Double-1> "${nmsp}::closeCall {$w} {$vx} {$c}"
	     }
             $w bind $j <Double-1> "${nmsp}::closeCall {$w} {$vx} {$c}"
             $w bind $j1 <Double-1> "${nmsp}::closeCall {$w} {$vx} {$c}"
	 } else {
             $w bind $j <1> "${nmsp}::closeCall {$w} {$vx} {$c}"
	     if {[info exists k]} {
               $w bind $k <1> "${nmsp}::closeCall {$w} {$vx} {$c}"
	     }
	 }
          buildlayer $w $vx/$c [expr $in+18] $graybar
      } else {
         set j [$w create image $in $y -image  closedbm -tags $taglist]
	 if {$Tree($w:windowsMode)} {
	     if {[info exists k]} {
                 $w bind $k <Double-1> "${nmsp}::openCall {$w} {$vx} {$c}"
	     }
             $w bind $j <Double-1> "${nmsp}::openCall {$w} {$vx} {$c}"
             $w bind $j1 <Double-1> "${nmsp}::openCall {$w} {$vx} {$c}"
	 } else {
             $w bind $j <1> "${nmsp}::openCall {$w} {$vx} {$c}"
	 }
      }
    }
  }
  if {[info exists j1]} {
    set ll2 [$w create line $in $start $in [expr $y+1] -fill gray50 ]
    $w raise $ll2 $j1
  }

}

proc openCall {w vx c } {
        variable Tree

        set Tree($w:$vx/$c:open) 1
        build $w;
        if {[info exists Tree(openCallBack)]} { 
	     eval $Tree(openCallBack) [list $vx/$c]
        }
}

proc closeCall {w vx c } {
        variable Tree

        set Tree($w:$vx/$c:open) 0
        build $w;
        if {[info exists Tree(closeCallBack)]} { 
	     eval $Tree(closeCallBack) [list $vx/$c]
        }
}

# Open a branch of a tree
#
proc open {w v} {
  variable Tree
  variable Tree_children

  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==0
      && [info exists Tree_children_$v)] 
      && [string length $Tree_children_$v)]>0} {($w($w
    set Tree($w:$v:open) 1
     [namespace current]::build $w
  } 
}

proc close {w v} {
  variable Tree
  if {[info exists Tree($w:$v:open)] && $Tree($w:$v:open)==1} {
    set Tree($w:$v:open) 0
     [namespace current]::build $w
  }
}

#
# Change the selection to the indicated item
#
proc setselection {w v} {
  variable Tree

  if {[set p [lsearch $Tree($w:selection) $v]] >= 0} {
    unsetselection $w $p
  } else {
    if {$Tree($w:single)} {
      unsetselection $w
    }
    lappend Tree($w:selection) $v
    drawselection $w
  }
}

# 
# Retrieve the current selection
#
proc getselection w {
  variable Tree
  return $Tree($w:selection)
}

proc unsetselection {w {p {-1}}} {
  variable Tree
  
  if {$p < 0} {
    foreach i $Tree($w:selidx) {
      $w delete $i
    }
    set Tree($w:selidx) {}
    set Tree($w:selection) {}
  } else {
     set i [lindex $Tree($w:selidx) $p]
     $w delete $i
     set Tree($w:selection) [lreplace $Tree($w:selection) $p $p]
     set Tree($w:selidx) [lreplace $Tree($w:selidx) $p $p]
  }
}

# Internal use only.
# Draw the selection highlight
proc drawselection w {
  variable Tree
  
  foreach i $Tree($w:selidx) {
    $w delete $i
  }

  set Tree($w:selidx) {}
  
  foreach v $Tree($w:selection) {
    if {[string length $v]==0} return
    if {![info exists Tree($w:$v:tag)]} return
    set bbox [$w bbox $Tree($w:$v:tag)]
    if {[llength $bbox]==4} {
      set i [eval $w create rectangle $bbox -fill yellow -outline {{}}]
      lappend Tree($w:selidx) $i
      $w lower $i
    }
  }
}

# Internal use only
# Call  build then next time we're idle
proc buildwhenidle w {
  variable Tree
  if {![info exists Tree($w:buildpending)]} {
    set Tree($w:buildpending) 1
    after idle " [namespace current]::build $w"
  }
}

#
# Return the full pathname of the label for widget $w that is located
# at real coordinates $x, $y
#
proc labelat {w x y} {
  set x [$w canvasx $x]
  set y [$w canvasy $y]
  variable Tree
  foreach m [$w find overlapping $x $y $x $y] {
    if {[info exists Tree($w:tag:$m)]} {
      return $Tree($w:tag:$m)
    }
  }
  return ""
}

proc getImageAt {w x y } {
  set x [$w canvasx $x]
  set y [$w canvasy $y]
  variable Tree
  foreach m [$w find overlapping $x $y $x $y] {
    if {[info exists Tree($w:tag:$m)]} {

      if {![catch {$w itemcget $m -image} imgName]} {
      } else {
          set im [expr $m - 1]
          if {[catch {$w itemcget $im -image} imgName]} {
	      set imgName "noImg"
	  }
      }
      return $imgName
    }
  }
  return ""
}

}



#################
#
# The remainder is code that demonstrates the use of the Tree
# widget.  
#

image create photo idir -data {
    R0lGODdhEAAQAPIAAAAAAHh4eLi4uPj4APj4+P///wAAAAAAACwAAAAAEAAQAAADPVi63P4w
    LkKCtTTnUsXwQqBtAfh910UU4ugGAEucpgnLNY3Gop7folwNOBOeiEYQ0acDpp6pGAFArVqt
    hQQAO///
}

image create photo ifile -data {
    R0lGODdhEAAQAPIAAAAAAHh4eLi4uPj4+P///wAAAAAAAAAAACwAAAAAEAAQAAADPkixzPOD
    yADrWE8qC8WN0+BZAmBq1GMOqwigXFXCrGk/cxjjr27fLtout6n9eMIYMTXsFZsogXRKJf6u
    P0kCADv/
}

if {[info exists argv] && ([lsearch $argv -test] >= 0)} {

image create photo equip1 -data {
R0lGODlhGQATAPZpAAAAAAADAwADBAAFBQAGBQAICAANDQAREQASEgAUFAAVFAAfHwAgHwAm
JQAmJgApKAApKQAvLwAwLwAwMAAyMQAyMgA0NAA2NgA3NwA4NwA4OAA5OAA8OwA9PAA/PgA/
PwBBQABDRABEQgBGRQBHRgBJRwBKSQBLSgBPTgBPTwBQTgBQTwBYVwBzcgCFgwCOjACPjQCQ
jQCRjQCamACbmQCcmgCfngCjoQCqpgCtqQCtqgCurACyrgC3swDHxADNyQDOywDPzADX1ADZ
1gDc2ADi3QDn4wDp5ADu6gDv6wDy7gD87wDz8AD18QD28wD38gD39AD49AD59AD79wD88AD8
9wD99wT88wD8+AD8+QD9+QH8+AD++gD++wD//AD//QD//gD//wn8+An9+Qn++gr8+Ar9+Qn/
/Qn//xH8+AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAACH5BAAAAGoALAAAAAAZABMAAAf+gGlmZISEY2KIiYqLiGVh
j5CPWJNYWpGXWltoPQYLDAwIKmFalF9QJyIlqiImTV9YaDkAs7MXl4+0uUxhsDy5AAQkGhkZ
GBwev7NRvGi+uQEeExLTFhvJAMu9vx23YQPJTsw4vw1hXJRhStdJvGc+HygpKyEto5RcYEVH
VhQyUkXmsIjpFuaLwS9evkCKcOPROSxXslCaRPBcmAczwkyhtGQiRRYCFCQ4UEAIrzAQaGjk
6BFLGBC/TFpKuXJSR49hRvwyAqlCjZpYbk7MSc7BAggAbISpwhKnzlw7hvwIAuQJr6ZDL/wi
EukLKazodLiIAeOFCyQaN7akwtRjFy8HcOF+bTkpEAA7
}

image create photo equip2 -data {
R0lGODlhGQATAPZVAAAAAAIBAAICAAcGAA8OABARABUWABwdACEhACMkADc3ADk7AD0/AD1A
AFNTAF1gAGNjAGNmAHByAHV1AHZ4AH+CAACcmACinwCjoQCmpACnpQDAvADCvwDJxgDLxwDg
3QDi3gDo5QDp5gDq5gDq5wDs6QDt6ADu6QD87wD08QD18gD79wD88AD89wT88wD8+AD8+QD9
+QH8+AD++QD++gD//AD//QD//wn8+Ar8+BH8+JyeAKaoAKiqAKqsAK6wALS5ALm8ALq+ALzA
AMnLAM3QANHUAN3gAN/hAN7iAN/kAO/yAPf7APf8APj7APj8APj9APn9APr+AP3/AP7/AP//
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAACH5BAAAAFYALAAAAAAZABMAAAfogDo5OISFhoeIhDkvjCuO
j5CRj4wyOC81N5mam5ycNC84NjMfIiOmp6ipJCEgN6A3HACys7S1tSY0ODcdtr29Kbk3HrYE
DRAUEw8KA7YqwcOzDkRV1NVVVEEMtM660AJC1VFPTk9PUtU7s9zCsj5VU+Xx8k/UFbLOLuwB
VFVM8/NMqiC5RwMFOwBHqkD5Jy9KFSAEDUJbQI3KEyYYMTZxWGXJgYgHASQYYq1klR4F1BUM
KQuBhB9GlCQpwiOCgVrOJPraqdLgBp48S6w8YSGDhqNIkyrVgOECjBgsWsSoQbWq1atXYbwI
BAA7
}

image create photo locat1 -data {

R0lGODlhGQATAPZhAAAAAAMDAAADBQAFBgAGBgAUEwAUFQAVFgAXGAAcHQAeHgBPAABQAABZ
AABcAABeAABgAABkAABkDQB3AAB4AAB4HABfXwBkawBqaABubABzcQB1cwB2dAB4cQB4dgB5
dwB7d0FBQURFREtNS0xOTACfAACuAACAfgDyAADzAAD6AAD7AAD8AAD/AAD/LgDzNGC6AACD
gACTkQmkogmlowm8ugm+uwrSwgnZ1QnZ1gna1wDz1AHw3ADz3gDz3wD08AH59QD8+AD9+QD9
+wD++gD/+gD/+wD//AD//4KEAI+TAJ6KAJiaAJ73ALr/ANXXAPL2AP/wAPr+APz/AP//AKCj
oKmtqsXJxcbLxtfa19fb1+Tn5OPo5OTo5OXp5eru6vL28/j8+P///wAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAACH5BAAAAGMALAAAAAAZABMAAAf0gDc4OYQ6NjM0NDM1OoSO
j448R0ZFREgxAAUFACBIREWgoaJAPUNCQUFIGgCsABhIqLGysaVGR0dIHq0AGUi3v8C3Qj1I
xcUnuxzGy8w/xBsDCAoEuwMJBtjZ2QcCMj5IFrvi4+MfO0gXACJhXVvuW13x8vJcYFYAHS8u
EgAjYv8AAwr8dwVAhRQtIgAgMbBhQCwAJiBUGOKLliwYM2rMouWilyoRJ5IbOY6CSJIoAZhM
mHJXEilPArBaqbAlKyZUprSiaZOVEipQdiJs0BPAEipRhLYowQCC06dQny6A4aTJAwcMTKBQ
waKr169gWawY61VFIAA7
}

image create photo locat2 -data {
R0lGODlhGQATAPZcAAAAAAUAAwYAAAcABggHCAgICA8PDxUAAB4AABAQABEREQApBgA0AAA3
AAA5AAA+AAA/AAAhIQAiIgAlJAAnJwAiLSoODisODjMXFzUZGTg6AAhHAABIEwBQAABYAABd
AABeAABnACVnAABCQABDQQBHRQBHRwBDUH5mZH9nZQD5AAD8AAD9AAD/AAD8DQD8GgD8bAD8
bQD8bgD9bgD8cgD/egD/fQCHhgCJhwCKiQCLigCMigCxrwC/vAD8gAnq1grq1hHo2gH79QD8
9wD8+AD8+QD9+AD9+QD8+wD++gD8/wD9/wD//QD//wX8+Ar79Qn8+IFtAIJqaoJqdopzcf//
AJ+NiaCOiqCOlqORkayZl/f79/j8+Pn9+fr++v7//v///wAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA
AAAAAAAAAAAAAAAAAAAAAAAAACH5BAAAAGEALAAAAAAZABMAAAf1gEFAP4SFhoeIhE9JjElM
TZCRkk1MjY1CUEWaRE09Nzs5oaE6ODxNmqhFRFBErZwlBxkWF7QWGAgTTa67TrtNJlRgwsNg
WhRNR7vKrU0kKWBdXtJdYFcSyMtEQ9tDTSMoYFtc41tgVhFNRtzbREhK70pNJ1LEw1kVTUvw
70g+MDEyYtjgMAXMl2EHsSyoMUOGQ4c0XqyYuKKFhwAKCBTYSMDAgAYtWFCk6GJkiw8AUqpU
+SDkyJcTW4BYSdOBS5gmZ9IEkCDKhps4Y+qkqaGKCKBBZe5MKYABUpxKlwJoKTKoUKkAIDyF
GXWp1qpWWYTA2kEFzkAAOw==
}

image create photo equip1 -data {
R0lGODdhFgAQAKEAAPj8+AAAAPj8APj4+CwAAAAAFgAQAAACO4QPoYu9D1U7scqJntjiYapw
HPMlodhd34KmQWkGLbmeIo2xKD5po/XKACs9C0wI+QyWzMHx0IxKp4MCADs=
}

image create photo equip1 -data {
R0lGODdhFgAQAMIAAPj8+PgAAAAA+AAAAAD8APj4+AAAAAAAACwAAAAAFgAQAAADTAi63P4Q
hkkDFDiLSq/GzeRFwDSc6Ak+50S8yktgp9NaMgzQg02kQNTPlyvmeo3BS8n8OZlE51KmHCZP
xmONFQQ2CuBwgcQQm8/oQgIAOw==
}

image create photo locat1 -data {
R0lGODdhFgAQAKEAAPj8+AAAAPj8APj4+CwAAAAAFgAQAAACOISPFxvpz9qbJqaIcwa670VN
4NhBzSikqiBp3LmqrUy+LkefUhjktsfwATHCoSeENAyWzKbzOSgAADs=
}


set Ogui(background) gray90

set f [frame .f]
pack .f

   Tree::create $f.w -width 250 -height 340 -yscrollcommand "$f.sb set" \
      -background $Ogui(background)
   scrollbar $f.sb -orient vertical -command "$f.w yview"
   grid $f.w  -row 0 -column 1
   grid $f.sb -row 0 -column 0 -sticky ns

set st [clock seconds]   
   for {set i 0} {$i < 10} {incr i} {
      Tree::newitem $f.w /l1_$i -image idir -text "Directory_$i"
      for {set j 0} {$j < 10} {incr j} {
          Tree::newitem $f.w /l1_$i/l2_$j -image idir
             for {set k 0} {$k < 10} {incr k} {
                  Tree::newitem $f.w /l1_$i/l2_$j/l3_$k -image idir
             }
      }
   }
 puts "elapsed [expr [clock seconds] - $st]"

   Tree::create $f.w2 -width 250 -height 200 \
      -background #eee \
      -windowsMode 1 \
      -graybar 1 \
      -graycolor #ffd \
      -highlight 1\
      -highlightcolor #8f8

  grid $f.w2 -row 1 -column 1
  array set bldg {0 {Admin Bldg} 1 {Manufacturing} 2 {Quality Control} 3 {Shipping} }
  array set equip {0 zero 1 one 2 two 3 three 4 four}
  for {set i 0} {$i < 3} {incr i} {
        ::Tree::newitem $f.w2 /$i -text $bldg($i) -image locat1
     for {set j 0} {$j < 4} {incr j} {
        ::Tree::newitem $f.w2 /$i/$j -text $equip($j) -bind "\
	    tk_messageBox -message {I'm $i/$j} -type ok" \
	    -image equip1
     }
  }

  toplevel .t
  Tree::create .t.tree 
  Tree::newitem .t.tree /
  Tree::newitem .t.tree /xx
  Tree::newitem .t.tree /yy
  Tree::newitem .t.tree /zz
  Tree::newitem .t.tree /zz/aa
  Tree::newitem .t.tree /zz/bb
  Tree::newitem .t.tree /zz/cc
  catch {Tree::newitem .t.tree /xx}
  grid .t.tree

}
